@@ -20,7 +20,9 @@
 
 #include "networkstatus.h"
 #include "uiutils.h"
+
 #include <QDBusConnection>
+#include <QDBusConnectionInterface>
 
 #include <NetworkManagerQt/ActiveConnection>
 #include <NetworkManagerQt/Connection>
@@ -72,6 +74,7 @@ NetworkStatus::SortedConnectionType NetworkStatus::connectionTypeToSortedType(Ne
 NetworkStatus::NetworkStatus(QObject* parent)
     : QObject(parent)
 {
+    connect(NetworkManager::notifier(), &NetworkManager::Notifier::connectivityChanged, this,  &NetworkStatus::changeActiveConnections);
     connect(NetworkManager::notifier(), &NetworkManager::Notifier::statusChanged, this, &NetworkStatus::statusChanged);
     connect(NetworkManager::notifier(), &NetworkManager::Notifier::activeConnectionsChanged, this, QOverload<>::of(&NetworkStatus::activeConnectionsChanged));
 
@@ -111,6 +114,7 @@ void NetworkStatus::defaultChanged()
 
 void NetworkStatus::statusChanged(NetworkManager::Status status)
 {
+    const auto oldNetworkStatus = m_networkStatus;
     switch (status) {
         case NetworkManager::ConnectedLinkLocal:
             m_networkStatus = i18nc("A network device is connected, but there is only link-local connectivity", "Connected");
@@ -142,12 +146,14 @@ void NetworkStatus::statusChanged(NetworkManager::Status status)
         status == NetworkManager::ConnectedSiteOnly ||
         status == NetworkManager::Connected) {
         changeActiveConnections();
-    } else {
+    } else if (m_activeConnections != m_networkStatus) {
         m_activeConnections = m_networkStatus;
         Q_EMIT activeConnectionsChanged(m_activeConnections);
     }
 
-    Q_EMIT networkStatusChanged(m_networkStatus);
+    if (oldNetworkStatus != m_networkStatus) {
+        Q_EMIT networkStatusChanged(m_networkStatus);
+    }
 }
 
 void NetworkStatus::changeActiveConnections()
@@ -210,7 +216,20 @@ void NetworkStatus::changeActiveConnections()
                 if (connecting) {
                     status = i18n("Connecting to %1", connection->name());
                 } else if (connected) {
-                    status = i18n("Connected to %1", connection->name());
+                    switch (NetworkManager::connectivity()) {
+                        case NetworkManager::NoConnectivity:
+                            status = i18n("Connected to %1 (no connectivity)", connection->name());
+                            break;
+                        case NetworkManager::Limited:
+                            status = i18n("Connected to %1 (limited connectivity)", connection->name());
+                            break;
+                        case NetworkManager::Portal:
+                            status = i18n("Connected to %1 (log in required)", connection->name());
+                            break;
+                        default:
+                            status = i18n("Connected to %1", connection->name());
+                            break;
+                    }
                 }
 
                 if (!activeConnections.isEmpty()) {
@@ -223,8 +242,10 @@ void NetworkStatus::changeActiveConnections()
         }
     }
 
-    m_activeConnections = activeConnections;
-    Q_EMIT activeConnectionsChanged(activeConnections);
+    if (m_activeConnections != activeConnections) {
+        m_activeConnections = activeConnections;
+        Q_EMIT activeConnectionsChanged(activeConnections);
+    }
 }
 
 QString NetworkStatus::checkUnknownReason() const
